/*
 * Copyright (C) 2000-2011 OSS Nokalva.  All rights reserved.
 *
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA AND MAY BE USED ONLY
 * BY DIRECT LICENSEES OF OSS NOKALVA. THIS FILE MAY NOT BE DISTRIBUTED.
 */
 /*
 * This simple C program  illustrates use of the file memory manager.  The
 * ASN.1 syntax does not use the OBJHANDLE directive (see BCASFL.ASN). The file
 * manager reads from and writes to a file.
 */

#include <stdio.h>
#include <string.h>
#include "bcasfl.h"

#ifdef CHECK_FOR_MEMORY_PROBLEMS
extern void *DLL_ENTRY getmem(size_t);
extern void  DLL_ENTRY rlsemem(void *);
extern void *DLL_ENTRY _getmem(size_t);
extern void  DLL_ENTRY _rlsmem(void *);
extern void  memoryReport(void);
#endif /* CHECK_FOR_MEMORY_PROBLEMS */

int main(void)
{

    OssGlobal w, *world = &w;   /* structure ossGlobal */
    OssBuf    encodedData;      /* length and address of encoded data */
    BBCard   *myCardPtr = NULL; /* address of decoded data */
    int	      pdu_num = 0;      /* PDU number */
    int	      i, retcode;       /* return code */
    int	      num = 11, j;
    char      encoderFile[32];  /* encoded data file name */


	/*
	 * Call ossinit() first to initialize OSS's global structure
	 */
    if (retcode = ossinit(world, bcasfl)) {
	printf("Ossinit() returned %d\n", retcode);
	return retcode;
    }

#ifdef CHECK_FOR_MEMORY_PROBLEMS
#ifndef _WINDOWS
    world->mallocp = (void *(DLL_ENTRY_FPTR *)(size_t))getmem;
    world->freep   = (void  (DLL_ENTRY_FPTR *)(void *))rlsemem;
#else
    world->mallocp = (void *(DLL_ENTRY_FPTR *)(size_t))_getmem;
    world->freep   = (void  (DLL_ENTRY_FPTR *)(void *))_rlsmem;
#endif /* _WINDOWS */
#endif /* CHECK_FOR_MEMORY_PROBLEMS */

    ossSetEncodingFlags(world, DEBUGPDU);
    ossSetDecodingFlags(world, DEBUGPDU);
    strcpy(encoderFile, "catpdus.dat");
	/*
	 * Print the input to the encoder
	 */
    ossPrint(world, "The input to the encoder ...\n\n");
    ossPrintPDU(world, BBCard_PDU, &myCard);
	/*
	 * Set the name of the encoder output file and mark it
	 * with the MARK_OBJECT macro
	 */
    encodedData.value = (unsigned char *)ossMarkObj(world, OSS_FILE, encoderFile);

    for (j = 0; j < 4; j++) {
	retcode = remove(encoderFile);

	ossSetEncodingRules(world, (ossEncodingRules)j);
	encodedData.length = 0;

	ossPrint(world, "\nCreating Concatenated %s-Encoded PDUs In A File...\n\n",
		j == OSS_BER ? "BER" : (j == OSS_PER_ALIGNED ? "PER_ALIGNED" :
		(j == OSS_PER_UNALIGNED ? "PER_UNALIGNED" : "DER")));

	if (retcode = ossOpenEncoderOutputFile(world, encoderFile)) {
	    ossPrint(world,"\nCan't open encoder output file \n\n");
	    ossterm(world);
	    return retcode;
	}

	for (i = 1; i < num && !retcode; i++) {
	    ossPrint(world, "\nEncoding PDU #%d ...\n\n", i);
	    if (retcode = ossEncode(world, BBCard_PDU, &myCard, &encodedData))
		ossPrint(world, "%s\n", ossGetErrMsg(world));
	}
	if (!(retcode = ossCloseEncoderOutputFile(world))) {

	    ossPrint(world,"\nThe decoder's trace messages ...\n\n");

	    if (retcode = ossOpenDecoderInputFile(world, encoderFile))
		ossPrint(world,"\nCan't open decoder input file \n\n");
	    else {
		/*
		 * Set up the decoder input file name so the decoder decodes
		 * from the file
		 */
		encodedData.length = 0;

		for (i = 1; !retcode; i++) {
		    ossPrint(world, "\nDecoding PDU #%d...\n\n", i);
		    if (retcode = ossDecode(world, &pdu_num, &encodedData,
						(void **)&myCardPtr))
			ossPrint(world, "%s\n", ossGetErrMsg(world));
		    else {
			encodedData.length += ossGetBytesReadByDecoder(world);
			/*
			 * Print the decoder's output
			 */
			ossPrint(world, "\nOutput from the decoder ...\n\n");
			ossPrintPDU(world, pdu_num, myCardPtr);
			ossFreePDU(world, pdu_num, myCardPtr);
			myCardPtr = NULL;
		    }
		}
		ossCloseDecoderInputFile(world);
	    }
	}
    }
    ossFreeBuf(world, encodedData.value);
    ossPrint(world, "\n");
    ossCloseEncoderOutputFile(world);
    ossCloseDecoderInputFile(world);
    ossterm(world);

#ifdef CHECK_FOR_MEMORY_PROBLEMS
    memoryReport();
#endif /* CHECK_FOR_MEMORY_PROBLEMS */

    return retcode;
}

